@@ -10,9 +10,13 @@
 #include "common.h"
 #include <sys/stat.h>
 #include <string.h>
-#include <X11/Xlib.h>
-#include <X11/extensions/XShm.h>
-#include <X11/Xutil.h>
+
+#ifndef X_DISPLAY_MISSING
+#  include <X11/Xlib.h>
+#  include <X11/extensions/XShm.h>
+#  include <X11/Xutil.h>
+#endif
+
 #include "image.h"
 
 char                load(ImlibImage * im, ImlibProgressFunction progress,
@@ -107,6 +111,11 @@ load(ImlibImage * im, ImlibProgressFunction progress,
       fseek(f, 8, SEEK_CUR);
       ReadleLong(f, &offset);
       ReadleLong(f, &headSize);
+      if (offset >= size)
+        {
+           fclose(f);
+           return 0;
+        }
       if (headSize == 12)
         {
            ReadleShort(f, &tmpShort);
@@ -135,6 +144,12 @@ load(ImlibImage * im, ImlibProgressFunction progress,
            fclose(f);
            return 0;
         }
+      
+      if ((w > 8192) || (h > 8192))
+	{
+	   fclose(f);
+	   return 0;
+	}
 
       if (bitcount < 16)
         {
@@ -142,12 +157,14 @@ load(ImlibImage * im, ImlibProgressFunction progress,
            if (headSize == 12)
              {
                 ncols /= 3;
+                if (ncols > 256) ncols = 256;
                 for (i = 0; i < ncols; i++)
                    fread(&rgbQuads[i], 3, 1, f);
              }
            else
              {
                 ncols /= 4;
+                if (ncols > 256) ncols = 256;
                 fread(rgbQuads, 4, ncols, f);
              }
         }
@@ -697,12 +714,14 @@ load(ImlibImage * im, ImlibProgressFunction progress,
    return 1;
 }
 
+#if 0
 char
 save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
 {
    /* TODO */
    return 0;
 }
+#endif
 
 void
 formats(ImlibLoader * l)
