@@ -8,9 +8,13 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <string.h>
-#include <X11/Xlib.h>
-#include <X11/extensions/XShm.h>
-#include <X11/Xutil.h>
+
+#ifndef X_DISPLAY_MISSING
+#  include <X11/Xlib.h>
+#  include <X11/extensions/XShm.h>
+#  include <X11/Xutil.h>
+#endif
+
 #include "image.h"
 #include <gif_lib.h>
 
@@ -217,15 +221,6 @@ load(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity,
    return 1;
 }
 
-char
-save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
-{
-   return 0;
-   im = NULL;
-   progress = NULL;
-   progress_granularity = 0;
-}
-
 /* fills the ImlibLoader struct with a strign array of format file */
 /* extensions this loader can load. eg: */
 /* loader->formats = { "jpeg", "jpg"}; */
