@@ -18,9 +18,14 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
-#include <X11/Xlib.h>
-#include <X11/extensions/XShm.h>
-#include <X11/Xutil.h>
+#include <sys/mman.h>
+
+#ifndef X_DISPLAY_MISSING
+#  include <X11/Xlib.h>
+#  include <X11/extensions/XShm.h>
+#  include <X11/Xutil.h>
+#endif
+
 #include "image.h"
 #include "colormod.h"
 #include "blend.h"
@@ -32,7 +37,7 @@ char                save(ImlibImage * im, ImlibProgressFunction progress,
 void                formats(ImlibLoader * l);
 
 /* flip an inverted image - see RLE reading below */
-static DATA32      *flip(DATA32 * in, int w, int h);
+static void         tgaflip(DATA32 * in, int w, int h);
 
 /* TGA pixel formats */
 #define TGA_TYPE_MAPPED      1
@@ -209,58 +214,65 @@ char
 load(ImlibImage * im, ImlibProgressFunction progress,
      char progress_granularity, char immediate_load)
 {
-   FILE               *fp;
+   int                 fd;
+   void                *seg, *filedata;
+   struct stat         ss;
    int                 bpp, vinverted = 0;
    int                 rle = 0, footer_present = 0;
 
-   tga_header          header;
-   tga_footer          footer;
+   tga_header          *header;
+   tga_footer          *footer;
 
    if (im->data)
       return 0;
 
-   fp = fopen(im->real_file, "rb");
-   if (!fp)
+   fd = open(im->real_file, O_RDONLY);
+   if (fd < 0)
       return 0;
 
-   /* read the footer first */
-   fseek(fp, 0L - (sizeof(tga_footer)), SEEK_END);
-   if (fread(&footer, sizeof(tga_footer), 1, fp) != 1)
+   if (fstat(fd, &ss) < 0)
      {
-        fclose(fp);
+        close(fd);
         return 0;
      }
 
+   if (ss.st_size < sizeof(tga_header) + sizeof(tga_footer))
+     {
+        close(fd);
+        return 0;
+     }
+   seg = mmap(0, ss.st_size, PROT_READ, MAP_SHARED, fd, 0);
+   if (seg == MAP_FAILED)
+     {
+        close(fd);
+	return 0;
+     }
+
+   filedata = seg;
+   header = (tga_header *) filedata;
+   footer = (tga_footer *) ((char *)filedata + ss.st_size - sizeof(tga_footer));
+
    /* check the footer to see if we have a v2.0 TGA file */
-   if (memcmp(footer.signature, TGA_SIGNATURE, sizeof(footer.signature)) == 0)
+   if (memcmp(footer->signature, TGA_SIGNATURE, sizeof(footer->signature)) == 0)
       footer_present = 1;
 
    if (!footer_present)
      {
-        fclose(fp);
-        return 0;
-     }
-
-   /* now read the header */
-   if (fseek(fp, 0, SEEK_SET) || fread(&header, sizeof(header), 1, fp) != 1)
-     {
-        fclose(fp);
-        return 0;
      }
 
+   /* skip over header */
+   filedata = (char *)filedata + sizeof(tga_header);
+   
    /* skip over alphanumeric ID field */
-   if (header.idLength && fseek(fp, header.idLength, SEEK_CUR))
-     {
-        fclose(fp);
-        return 0;
-     }
+   if (header->idLength)
+     filedata = (char *)filedata + header->idLength;
 
    /* now parse the header */
 
    /* this flag indicated bottom-up pixel storage */
-   vinverted = header.descriptor ^ TGA_DESC_VERTICAL;
+   vinverted = !(header->descriptor & TGA_DESC_VERTICAL);
 
-   switch (header.imageType)
+   switch (header->imageType)
      {
        case TGA_TYPE_COLOR_RLE:
        case TGA_TYPE_GRAY_RLE:
@@ -273,27 +285,30 @@ load(ImlibImage * im, ImlibProgressFunction progress,
           break;
 
        default:
-          fclose(fp);
-          return 0;
+	  munmap(seg, ss.st_size);
+	  close(fd);
+	  return 0;
      }
 
    /* bits per pixel */
-   bpp = header.bpp;
+   bpp = header->bpp;
 
    if (!((bpp == 32) || (bpp == 24) || (bpp == 8)))
      {
-        fclose(fp);
+	munmap(seg, ss.st_size);
+        close(fd);
         return 0;
      }
 
    /* endian-safe loading of 16-bit sizes */
-   im->w = (header.widthHi << 8) | header.widthLo;
-   im->h = (header.heightHi << 8) | header.heightLo;
+   im->w = (header->widthHi << 8) | header->widthLo;
+   im->h = (header->heightHi << 8) | header->heightLo;
 
    if ((im->w > 32767) || (im->w < 1) || (im->h > 32767) || (im->h < 1))
      {
         im->w = 0;
-        fclose(fp);
+	munmap(seg, ss.st_size);
+        close(fd);
         return 0;
      }
 
@@ -310,8 +325,7 @@ load(ImlibImage * im, ImlibProgressFunction progress,
    if (((!im->data) && (im->loader)) || (immediate_load) || (progress))
      {
         unsigned long       datasize;
-        struct stat         ss;
-        unsigned char      *buf, *bufptr;
+        unsigned char      *bufptr;
         DATA32             *dataptr;
 
         int                 y, pl = 0;
@@ -322,7 +336,8 @@ load(ImlibImage * im, ImlibProgressFunction progress,
         if (!im->data)
           {
              im->w = 0;
-             fclose(fp);
+	     munmap(seg, ss.st_size);
+             close(fd);
              return 0;
           }
 
@@ -332,29 +347,13 @@ load(ImlibImage * im, ImlibProgressFunction progress,
         /* find out how much data must be read from the file */
         /* (this is NOT simply width*height*4, due to compression) */
 
-        stat(im->real_file, &ss);
-        datasize = ss.st_size - sizeof(tga_header) - header.idLength -
+        datasize = ss.st_size - sizeof(tga_header) - header->idLength -
             (footer_present ? sizeof(tga_footer) : 0);
 
-        buf = malloc(datasize);
-        if (!buf)
-          {
-             im->w = 0;
-             fclose(fp);
-             return 0;
-          }
-
-        /* read in the pixel data */
-        if (fread(buf, 1, datasize, fp) != datasize)
-          {
-             fclose(fp);
-             return 0;
-          }
-
         /* buffer is ready for parsing */
 
         /* bufptr is the next byte to be read from the buffer */
-        bufptr = buf;
+        bufptr = filedata;
 
         /* dataptr is the next 32-bit pixel to be filled in */
         dataptr = im->data;
@@ -369,9 +368,9 @@ load(ImlibImage * im, ImlibProgressFunction progress,
                   /* point dataptr at the beginning of the row */
                   if (vinverted)
                      /* some TGA's are stored upside-down! */
-                     dataptr = im->data + (im->h - (y + 1)) * im->w;
+                     dataptr = im->data + ((im->h - y - 1) * im->w);
                   else
-                     dataptr = im->data + y * im->w;
+                     dataptr = im->data + (y * im->w);
 
                   for (x = 0; x < im->w; x++)   /* for each pixel in the row */
                     {
@@ -380,7 +379,8 @@ load(ImlibImage * im, ImlibProgressFunction progress,
 
                               /* 32-bit BGRA pixels */
                            case 32:
-                              WRITE_RGBA(dataptr, *(bufptr + 2),        /* R */
+                              WRITE_RGBA(dataptr,
+					 *(bufptr + 2), /* R */
                                          *(bufptr + 1), /* G */
                                          *(bufptr + 0), /* B */
                                          *(bufptr + 3)  /* A */
@@ -391,7 +391,8 @@ load(ImlibImage * im, ImlibProgressFunction progress,
 
                               /* 24-bit BGR pixels */
                            case 24:
-                              WRITE_RGBA(dataptr, *(bufptr + 2),        /* R */
+                              WRITE_RGBA(dataptr,
+					 *(bufptr + 2), /* R */
                                          *(bufptr + 1), /* G */
                                          *(bufptr + 0), /* B */
                                          (char)0xff     /* A */
@@ -402,41 +403,26 @@ load(ImlibImage * im, ImlibProgressFunction progress,
 
                               /* 8-bit grayscale */
                            case 8:
-                              WRITE_RGBA(dataptr, *bufptr,      /* grayscale */
-                                         *bufptr, *bufptr, (char)0xff);
+                              WRITE_RGBA(dataptr, /* grayscale */
+					 *bufptr,
+                                         *bufptr, 
+					 *bufptr, (char)0xff);
                               dataptr++;
                               bufptr += 1;
                               break;
                          }
 
                     }           /* end for (each pixel) */
-
-                  /* report progress every row */
-                  if (progress)
-                    {
-                       char                per;
-                       int                 l;
-
-                       per = (char)((100 * y) / im->h);
-                       if (((per - pper) >= progress_granularity) ||
-                           (y == (im->h - 1)))
-                         {
-                            l = y - pl;
-                            if (!progress(im, per, 0, (y - l), im->w, l))
-                              {
-                                 free(buf);
-                                 fclose(fp);
-                                 return 2;
-                              }
-                            pper = per;
-                            pl = y;
-                         }
-                    }
-
-               }                /* end for (each row) */
-
-          }                     /* end if (RLE) */
-
+	       }
+	     if (progress)
+	       {
+		  char                per;
+		  int                 l;
+		  
+		  progress(im, 100, 0, 0, im->w, im->h);
+	       } /* end for (each row) */
+          }
+	/* end if (!RLE) */
         /* decode RLE compressed data */
         else
           {
@@ -535,64 +521,23 @@ load(ImlibImage * im, ImlibProgressFunction progress,
                               }
                          }
                     }           /* end if (raw packet) */
-
-                  /* report progress every packet */
-                  if (progress)
-                    {
-                       char                per;
-                       int                 l;
-
-                       /* compute an approximate y value */
-                       /* can't be exact since packets don't necessarily */
-                       /* end at the end of a row */
-                       y = (dataptr - im->data) / im->w;
-
-                       per = (char)((100 * y) / im->h);
-
-                       if (((per - pper) >= progress_granularity) ||
-                           (y == (im->h - 1)))
-                         {
-                            l = y - pl;
-                            if (!progress(im, per, 0, (y - l), im->w, l))
-                              {
-                                 free(buf);
-                                 fclose(fp);
-                                 return 2;
-                              }
-                            pper = per;
-                            pl = y;
-                         }
-                    }           /* end progress report */
-
                }                /* end for (each packet) */
-
              /* must now flip a bottom-up image */
+             if (vinverted) tgaflip(im->data, im->w, im->h);
+	     if (progress)
+	       {
+		  char                per;
+		  int                 l;
+		  
+		  progress(im, 100, 0, 0, im->w, im->h);
+	       } /* end for (each row) */
+          }
+	/* end if (image is RLE) */
+     }
+   /* end if (loading pixel data) */
 
-             /* This is the best of several ugly implementations
-              * I considered. It's not very good since the image
-              * will be upside-down throughout the loading process.
-              * This could be done in-line with the de-RLE code
-              * above, but that would be messy to code. There's
-              * probably a better way... */
-
-             if (vinverted)
-               {
-                  im->data = flip(im->data, im->w, im->h);
-                  if (!im->data)
-                    {
-                       fclose(fp);
-                       free(buf);
-                       return 0;
-                    }
-               }
-
-          }                     /* end if (image is RLE) */
-
-        free(buf);
-
-     }                          /* end if (loading pixel data) */
-
-   fclose(fp);
+   munmap(seg, ss.st_size);
+   close(fd);
    return 1;
 }
 
@@ -613,31 +558,27 @@ formats(ImlibLoader * l)
 
 /**********************/
 
-/* flip a DATA32 image block vertically
- * by allocating a new block, then copying
- * the rows in reverse order
- */
+/* flip a DATA32 image block vertically in place */
 
-static DATA32      *
-flip(DATA32 * in, int w, int h)
+static void
+tgaflip (DATA32 * in, int w, int h)
 {
-   int                 adv, adv2, i;
-   DATA32             *out;
+   DATA32 *adv, *adv2;
+   int x, y;
 
-   out = malloc(w * h * sizeof(DATA32));
-   if (!out)
-      return NULL;
+   adv = in;
+   adv2 = in + (w * (h - 1));
 
-   adv = 0;
-   adv2 = w * h;
-
-   for (i = 0; i < h; i++)
+   for (y = 0; y < (h / 2); y++)
      {
+	DATA32 tmp;
+	for (x = 0; x < w; x++)
+	  {
+	     tmp = adv[x];
+	     adv[x] = adv2[x];
+	     adv2[x] = tmp;
+	  }
         adv2 -= w;
-        memmove(out + adv, in + adv2, w * sizeof(DATA32));
         adv += w;
      }
-
-   free(in);
-   return out;
 }
