@@ -7,9 +7,13 @@
 
 #include "common.h"
 #include <string.h>
-#include <X11/Xlib.h>
-#include <X11/extensions/XShm.h>
-#include <X11/Xutil.h>
+
+#ifndef X_DISPLAY_MISSING
+#  include <X11/Xlib.h>
+#  include <X11/extensions/XShm.h>
+#  include <X11/Xutil.h>
+#endif
+
 #include <setjmp.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -43,8 +47,6 @@ static void         put_separate_and_raster(TIFFRGBAImage *, uint32 *, uint32,
                                             unsigned char *);
 static void         raster(TIFFRGBAImage_Extra * img, uint32 * raster, uint32 x,
                            uint32 y, uint32 w, uint32 h);
-static void         error_handler(const char *module, const char *fmt,
-                                  va_list ap);
 char                load(ImlibImage * im, ImlibProgressFunction progress,
                          char progress_granularity, char immediate_load);
 char                save(ImlibImage * im, ImlibProgressFunction progress,
@@ -150,6 +152,7 @@ load(ImlibImage * im, ImlibProgressFunction progress,
    TIFFRGBAImage_Extra rgba_image;
    uint32             *rast = NULL;
    uint32              width, height, num_pixels;
+   char                txt[1024];
 
    if (im->data)
       return 0;
@@ -180,14 +183,20 @@ load(ImlibImage * im, ImlibProgressFunction progress,
    if (!tif)
       return 0;
 
-   if ((!TIFFRGBAImageOK(tif, "Cannot be processed by libtiff"))
-       || (!TIFFRGBAImageBegin((TIFFRGBAImage *) & rgba_image, tif, 0,
-                               "Error reading tiff")))
+   strcpy(txt, "Cannot be processed by libtiff");
+   if (!TIFFRGBAImageOK(tif, txt))
      {
         TIFFClose(tif);
         return 0;
      }
-
+   strcpy(txt, "Cannot begin reading tiff");
+   if (!TIFFRGBAImageBegin((TIFFRGBAImage *) & rgba_image, tif, 0,
+			   txt))
+     {
+        TIFFClose(tif);
+        return 0;
+     }
+   
    rgba_image.image = im;
    im->w = width = rgba_image.rgba.width;
    im->h = height = rgba_image.rgba.height;
@@ -250,18 +259,24 @@ load(ImlibImage * im, ImlibProgressFunction progress,
                   rgba_image.rgba.put.separate = put_separate_and_raster;
                }
           }
-
-        if (!TIFFRGBAImageGet((TIFFRGBAImage *) & rgba_image,
-                              rast, width, height))
-          {
-             _TIFFfree(rast);
-             free(im->data);
-             im->data = NULL;
-             TIFFRGBAImageEnd((TIFFRGBAImage *) & rgba_image);
-             TIFFClose(tif);
-
-             return 0;
-          }
+	if (rgba_image.rgba.samplesperpixel == 8)
+	  {
+	     if (!TIFFRGBAImageGet((TIFFRGBAImage *) & rgba_image,
+				   rast, width, height))
+	       {
+		  _TIFFfree(rast);
+		  free(im->data);
+		  im->data = NULL;
+		  TIFFRGBAImageEnd((TIFFRGBAImage *) & rgba_image);
+		  TIFFClose(tif);
+		  
+		  return 0;
+	       }
+	  }
+	else
+	  {
+	     printf("channel bits == %i\n", (int)rgba_image.rgba.samplesperpixel);
+	  }
 
         _TIFFfree(rast);
      }
@@ -283,7 +298,7 @@ save(ImlibImage * im, ImlibProgressFunction progress, char progress_granularity)
    DATA32              pixel, *data = im->data;
    double              alpha_factor;
    uint32              x, y;
-   uint8               r, g, b, a;
+   uint8               r, g, b, a = 0;
    int                 has_alpha = IMAGE_HAS_ALPHA(im);
    int                 i = 0, pl = 0;
    char                pper = 0;
