@@ -1,7 +1,9 @@
 #include "common.h"
+#ifndef X_DISPLAY_MISSING
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/extensions/XShm.h>
+#endif
 #include "colormod.h"
 #include "image.h"
 #include "scale.h"
@@ -73,7 +75,7 @@ __imlib_RenderGetPixel(Display * d, Drawable w, Visual * v, Colormap cm,
    else
      {
         unsigned int        rm, gm, bm;
-        int                 i, rshift, gshift, bshift;
+        int                 i, rshift = 0, gshift = 0, bshift = 0;
         DATA32              val;
 
         rm = v->red_mask;
